// -*- C++ -*-
/*!
* @file  BMS.cpp
* @brief BMS model
* @date $Date$
*
* $Id$
*/

#include "BMS.h"
#include <mclmcrrt.h>
#include <BMSLib.h>
#include "template.h"
#include <iostream>

// Module specification
// <rtc-template block="module_spec">
static const char* bms_spec[] =
{
	"implementation_id", "BMS",
	"type_name",         "BMS",
	"description",       "BMS model",
	"version",           "1.1.0",
	"vendor",            "Nakamura",
	"category",          "Saliency_Model",
	"activity_type",     "PERIODIC",
	"kind",              "DataFlowComponent",
	"max_instance",      "1",
	"language",          "C++",
	"lang_type",         "compile",
	// Configuration variables
	"conf.default.view", "1:ON",
	"conf.default.sod", "0:OFF",
	// Widget
	"conf.__widget__.view", "radio",
	"conf.__widget__.sod", "radio",
	// Constraints
	"conf.__constraints__.view", "(1:ON,0:OFF)",
	"conf.__constraints__.sod", "(1:ON,0:OFF)",
	""
};
// </rtc-template>

/*!
* @brief constructor
* @param manager Maneger Object
*/
BMS::BMS(RTC::Manager* manager)
	// <rtc-template block="initializer">
	: RTC::DataFlowComponentBase(manager),
	m_originalImageIn("image", m_originalImage),
	m_saliencyMapOut("saliencyMap", m_saliencyMap)

	// </rtc-template>
{
	StepwiseExecutionContextInit(m_pManager);
}

/*!
* @brief destructor
*/
BMS::~BMS()
{
}



RTC::ReturnCode_t BMS::onInitialize()
{
	// Registration: InPort/OutPort/Service
	// <rtc-template block="registration">
	// Set InPort buffers
	addInPort("image", m_originalImageIn);

	// Set OutPort buffer
	addOutPort("saliencyMap", m_saliencyMapOut);

	// Set service provider to Ports

	// Set service consumers to Ports

	// Set CORBA Service Ports

	// </rtc-template>

	// <rtc-template block="bind_config">
	// Bind variables and configuration variable
	bindParameter("view", m_view, "1:ON");
	bindParameter("sod", m_sod, "0:OFF");
	// </rtc-template>


	makeBMSdir(); 

	m_saliencyMapOut.initialize();

	return RTC::RTC_OK;
}


RTC::ReturnCode_t BMS::onFinalize()
{
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t BMS::onStartup(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BMS::onShutdown(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t BMS::onActivated(RTC::UniqueId ec_id)
{	
	prevflg = 0;

	return RTC::RTC_OK;
}


RTC::ReturnCode_t BMS::onDeactivated(RTC::UniqueId ec_id)
{
	switch_viewer(saliencyImageMat, m_saliencyMapOut.getName(), prevflg, 0);

	originalImageMat.release();
	saliencyImageMat.release();

	while(m_originalImageIn.isNew())
		m_originalImageIn.read();
	
	return RTC::RTC_OK;
}


RTC::ReturnCode_t BMS::onExecute(RTC::UniqueId ec_id)
{
	if (!m_originalImageIn.isNew())
	{
		STEPWISE_NOTIFY_NO_DATA();
		return RTC::RTC_OK;
	}

	m_originalImageIn.read(originalImageMat);
	saliencyMap(saliencyImageMat,originalImageMat);
	m_saliencyMapOut.write(saliencyImageMat);

	switch_viewer(saliencyImageMat, m_saliencyMapOut.getName(), prevflg, m_view);

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t BMS::onAborting(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BMS::onError(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BMS::onReset(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BMS::onStateUpdate(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BMS::onRateChanged(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/



extern "C"
{

	void BMSInit(RTC::Manager* manager)
	{
		coil::Properties profile(bms_spec);
		manager->registerFactory(profile,
			RTC::Create<BMS>,
			RTC::Delete<BMS>);
	}

};


void BMS::saliencyMap(cv::Mat& saliencyImageM, cv::Mat originalImageM)
{
	mwArray saliencyImage;
	mwArray originalImage;
	mwArray sod(m_sod);

	try{
		originalImageM >> originalImage;
		BMSSaliencyModel(1,saliencyImage,originalImage,sod);
		saliencyImage >> saliencyImageM;
	}
	
	catch(const mwException& e){
		std::cerr << e.what() << std::endl;
	}
	catch(...){
		std::cerr << "\ʃG[" << std::endl;
	}
	
}


void BMS::switch_viewer(const cv::Mat& src, const std::string name, int& prev, const int curr)
{
	if(curr != prev)	//flg changed
	{	
		if(prev == 1)
		{		
			prev = 0;
			std::cout << name << " viewer OFF" << std::endl;
			cv::destroyWindow(name);
		}
		else
		{		
			prev = 1;
			std::cout << name << " viewer ON" << std::endl;
			cv::namedWindow(name, CV_WINDOW_AUTOSIZE);
		}
	}	
	if(curr == 1){
		#if (!defined WIN32) || (!defined WIN64)
   			cvStartWindowThread();
		#endif
		cv::imshow(name,src);
		cv::waitKey(1);
	}
}